define(['BaseView', 'Model', 'Store', 'text!TemplatesExpertview', 'text!TemplatesArticleLoadingMore', 'text!TemplatesActivityLoadingMore', 'utilLazyLoad', 'iScroll'], function(BaseView, models, store, tplExpertview, tplArticleLoadingMore, tplActivityLoadingMore, utilLazyLoad) {

    var getExpertDetailModel = models.getExpertDetailModel;
    var getArticleListsModel = models.getArticleListsModel;
    var getActivityListsModel = models.getActivityListsModel;
    var myScroll;

    var view = BaseView.extend({
        viewName: 'expertview',
        userid: null,
        username: null,
        _curArticlePage: 1,
        _totalArticlePage: 1,
        _curActivityPage: 1,
        _totalActivityPage: 1,
        events: {
            'click .js_toArticleDetail': 'toArticleDetail',
            'click .js_activityItem': 'toActivityDetail',
            'click .js_tab .ds_lis': 'toggleTab',
            // 'click .loadingMoreArticle': 'loadingMoreArticle',
            // 'click .loadingMoreActivity': 'loadingMoreActivity',
        },
        render: function(html, parent) {
            if (parent) {
                parent.append(html);
                return;
            }
            this.$el.html(html);
        },
        onShow: function() {
            var tempData = null;

            this.userid = Turtle.getQuery('userid');
            this.username = Turtle.getQuery('username');

            this.showLoading();
            getExpertDetailModel.setParam({
                user_id: this.userid
            });
            getArticleListsModel.setParam({
                pageSize: 99,
                curPage: 1,
                username: this.username
            });
            getActivityListsModel.setParam({
                pageSize: 99,
                curPage: 1,
                username: this.username
            });

            getExpertDetailModel.excute(function(json_expert) {
                getArticleListsModel.excute(function(json_article) {
                    getActivityListsModel.excute(function(json_activity) {
                        this.hideLoading();
                        json_expert = this.convertData('expert', json_expert.result);
                        json_article = this.convertData('article', json_article.result);
                        json_activity = this.convertData('activity', json_activity.result);
                        //因为是三个接口，所以要合并数据，一次性给模板去用
                        tempData = _.extend(json_expert, json_article, json_activity);
                        this.render(_.template(tplExpertview)(tempData));

                        this._curArticlePage = json_article.article_curPage;
                        this._totalArticlePage = json_article.article_totalPage;
                        this._curActivityPage = json_activity.activity_curPage;
                        this._totalActivityPage = json_activity.activity_totalPage;

                        // this.initScroll();

                        this.lazyLoad();

                    }, function(msg) {
                        this.hideLoading();
                        this.jump(['views/error.html?from=views/', this.viewName, '.html'].join(''));
                    }, false, this);
                }, function(msg) {
                    this.hideLoading();
                    this.jump(['views/error.html?from=views/', this.viewName, '.html'].join(''));
                }, false, this);

            }, function(msg) {
                this.hideLoading();
                this.jump(['views/error.html?from=views/', this.viewName, '.html'].join(''));
            }, false, this);
        },
        /**
         * 为三个数据合并进行准备，这样同名key不会相互覆盖
         * @param  {string} prefix 前缀
         * @param  {json data} data   
         * @return {json data} 所有key都加过前缀后的数据
         */
        convertData: function(prefix, data) {
            var rtnData = {};
            for (var key in data) {
                rtnData[prefix + '_' + key] = data[key];
            }
            return rtnData;
        },

        /**
         * Tab 切换文章和活动
         * data-classname 和 tab content 的 class 同名
         * @param  {eventobj} e    tab 按钮
         */
        toggleTab: function(e) {
            var zpItem = $(e.currentTarget);
            var contentClass = zpItem.data('classname');
            zpItem.addClass('on').siblings().removeClass('on');
            var zpContent = this.$el.find('.' + contentClass);
            zpContent.removeClass('none').siblings('.js_listWrap').addClass('none');
        },
        //已经废弃
        loadingMoreArticle: function() {
            if (this._curArticlePage < this._totalArticlePage) {
                getArticleListsModel.setParam({
                    pageSize: 3,
                    curPage: this._curArticlePage + 1,
                    username: this.username
                });
                getArticleListsModel.excute(function(json) {
                    this.render(_.template(tplArticleLoadingMore)(json.result), this.$el.find('.js_artList'));
                    this._curArticlePage = json.result.curPage;
                    this._totalArticlePage = json.result.totalPage;
                    if (this._curArticlePage == this._totalArticlePage) {
                        this.$el.find('.art_none').removeClass('none');
                    }
                }, function(msg) {
                    console.log('error');
                }, false, this);
            }
        },
        //已经废弃
        loadingMoreActivity: function() {
            if (this._curActivityPage < this._totalActivityPage) {
                getActivityListsModel.setParam({
                    pageSize: 3,
                    curPage: this._curActivityPage + 1,
                    username: this.username
                });
                getActivityListsModel.excute(function(json) {
                    this.render(_.template(tplActivityLoadingMore)(json.result), this.$el.find('.js_activityList'));
                    this._curActivityPage = json.result.curPage;
                    this._totalActivityPage = json.result.totalPage;
                    if (this._curActivityPage == this._totalActivityPage) {
                        this.$el.find('.art_none').removeClass('none');
                    }
                }, function(msg) {
                    console.log('error');
                }, false, this);
            }
        },
        initScroll: function() {
            var self = this;
            var $elWrapper = $('#wrapper');
            var loadStatus = true; //只有每当ajax发送加载完成后才能继续发送ajax
            var isAllLoaded = false; //全部页数都加载完成不用继续加载
            var pullUpEl;
            var pullUpOffset;

            if (myScroll) {
                myScroll.destroy();
            }
            //当前只有一页的情况立刻关闭iscroll
            if (self._curArticlePage >= self._totalArticlePage) {
                // $elWrapper.css({
                //     'overflow': 'initial'
                // });
                //修达人详情图片bug的代码
                $elWrapper.css({
                    'left': 0
                });
                return;
            }

            function pullUpAction() {
                if (loadStatus) {
                    loadStatus = false;
                    setTimeout(function() {
                        // ajax
                        if (self._curArticlePage < self._totalArticlePage) {
                            getArticleListsModel.setParam({
                                pageSize: 3,
                                curPage: self._curArticlePage + 1
                            });
                            getArticleListsModel.excute(function(json) {
                                self.render(_.template(tplArticleLoadingMore)(json.result), self.$el.find('.js_list'));
                                self._curArticlePage = json.result.curPage;
                                self._totalArticlePage = json.result.totalPage;
                                if (self._curArticlePage == self._totalArticlePage) {
                                    isAllLoaded = true;
                                    self.$el.find('.art_none').removeClass('none');
                                }
                                loadStatus = true;
                                myScroll.refresh();
                            }, function(msg) {
                                console.log('error');
                            }, false, self);
                        }
                    }, 1000);
                }
            }

            function loaded() {
                pullUpEl = document.getElementById('pullUp');
                pullUpOffset = pullUpEl.offsetHeight;

                myScroll = new iScroll('wrapper', {
                    useTransition: true,
                    onRefresh: function() {
                        if (pullUpEl.className.match('_loading')) {
                            pullUpEl.className = '';
                            pullUpEl.querySelector('.pullUpLabel').innerHTML = '';
                        }
                    },
                    onScrollMove: function() {
                        if (this.y < (this.maxScrollY - 5) && !pullUpEl.className.match('flip') && loadStatus) {
                            pullUpEl.className = 'flip';
                            pullUpEl.querySelector('.pullUpLabel').innerHTML = '';
                            this.maxScrollY = this.maxScrollY;
                        }
                    },
                    onScrollEnd: function() {
                        if (pullUpEl.className.match('flip') && loadStatus && !isAllLoaded) {
                            pullUpEl.className = '_loading';
                            pullUpEl.querySelector('.pullUpLabel').innerHTML = '加载中...';
                            pullUpAction();
                        }
                    }
                });
                setTimeout(function() {
                    document.getElementById('wrapper').style.left = '0';
                    $("#wrapper").css("top", $(".page").height() + 28); //修达人详情图片bug的代码
                }, 800);
            }
            loaded();
        },
        lazyLoad: function() {
            this.$el.find('.js_artlist img').picLazyLoad();
            this.$el.find('.js_activity img').picLazyLoad();
        },
        toArticleDetail: function(e) {
            var zpItem = $(e.currentTarget);
            var articleid = zpItem.data('articleid');
            var param = ['articleid=', articleid].join('');
            this.jump(['views/articleview.html?', param,
                '&from=views/', this.viewName, '.html',
                '?userid=', this.getQuery('userid'),
                '&username=', this.getQuery('username')
            ].join(''));
        },
        toActivityDetail: function(e) {
            var zpItem = $(e.currentTarget);
            var activityid = zpItem.data('activityid');
            var param = ['activityid=', activityid].join('');
            this.jump(['views/activityview.html?', param,
                '&from=views/', this.viewName, '.html',
                '?userid=', this.getQuery('userid'),
                '&username=', this.getQuery('username')
            ].join(''));
        },
        updateHeader: function() {
            var self = this;
            this.headerView.set({
                view: this,
                leftHandler: [{
                    className: "icon back",
                    event: function(event) {
                        self.back();
                    }
                }],
                centerHandler: [{
                    title: "达人详情",
                    className: "title"
                }],
                rightHandler: [{}]
            });
            this.headerView.show();
        }
    });

    this.viewIntance = view.getIntance();
});