define(['BaseView', 'Model', 'Store', 'text!TemplatesRegisterPhone', 'utilValidate', 'utilEncryption'], function(BaseView, models, store, tplRegisterPhone, utilValidate, utilEncryption) {

    var sendSmsCodeModel = models.sendSmsCodeModel;
    var mobileRegisterModel = models.mobileRegisterModel;
    var getAccessTokenModel = models.getAccessTokenModel;
    var getConfigModel = models.getConfigModel;
    var headStore = store.headStore;

    var view = BaseView.extend({
        viewName: 'register_phone',
        _dateTime: null, //从 sys_datetime 获取用做加密用
        _errorMsg: null,
        events: {
            'click .js_send_img_code': 'createImgCode',
            'click .js_send_code': 'sendSecurityCode',
            'click .js_register': 'register'
        },
        render: function(html) {
            this.$el.html(html);
        },
        onShow: function() {
            this.render(_.template(tplRegisterPhone)());
            this.getSiteConfig();
            this.createImgCode();
        },
        getSiteConfig: function() {
            this.showLoading();
            getConfigModel.setParam({
                client: 1
            });
            getConfigModel.excute(function(json) {
                this.hideLoading();
                //服务器返回的 "2015-06-24 11:08:53" 就用他们原来的代码改了
                var sysDatetime = json.result.sys_datetime;
                var aDateTime = sysDatetime.replace(" ", ":").replace(/\:/g, "-").split("-");
                this._dateTime = aDateTime[1] + aDateTime[2];
            }, function(msg) {
                this.hideLoading();
                if (_.isObject(msg) && msg.message) {
                    this.showToast(msg.message);
                    return;
                }
                this.showToast('网络错误，请稍候重试。');
            }, false, this);
        },
        /**
         * 图片验证码，其实是个假的，只是用JS模拟一段字符比较而已
         */
        createImgCode: function() {
            var zpSendImg = this.$el.find('.js_send_img_code');
            var codeLength = 4;
            var code = "";
            var selectChar = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'] //所有候选组成验证码的字符，当然也可以用中文的 
            for (var i = 0; i < codeLength; i++) {
                code += selectChar[Math.floor(Math.random() * selectChar.length)];
            }
            zpSendImg.val(code);
        },
        validateImgCode: function() {
            if (this.$el.find('.js_img_code').val() !== this.$el.find('.js_send_img_code').val()) {
                return false;
            }
            return true;
        },
        /**
         * 发送验证码
         */
        sendSecurityCode: function(e) {
            var zpItem = $(e.currentTarget);
            var oAllData = this.getAllVal();

            if (zpItem.hasClass('disable')) {
                return;
            }
            if (!utilValidate.isMobile(oAllData.mobileNum)) {
                this.showToast('手机号码格式不正确');
                return;
            }
            if (!this.validateImgCode()) {
                this.showToast('图片验证码输入错误！');
                this.createImgCode();
                return;
            }
            this.showLoading();
            sendSmsCodeModel.setParam({
                username: oAllData.mobileNum
            });
            sendSmsCodeModel.excute(function(json) {
                this.hideLoading();
                this.showToast('验证码发送成功');
                zpItem.addClass('disable');
                this.countDown(60);
                this.createImgCode();
            }, function(msg) {
                this.hideLoading();
                this.createImgCode();
                if (_.isObject(msg) && msg.message) {
                    this.showToast(msg.message);
                    return;
                }
                this.showToast('网络错误，请稍候重试。');
            }, false, this);

        },
        /**
         * 手机注册
         */
        register: function() {
            var self = this;
            var oAllData = this.getAllVal();
            if (this._isAllValValid(oAllData)) {
                this.showLoading();
                mobileRegisterModel.setParam({
                    channel: "user_channel_mobile", // varchar(255) 注册渠道
                    username: oAllData.mobileNum,
                    code: oAllData.securityCode, // char(4)      手机短信验证码
                    userpwd: oAllData.password, // varchar(6-20)    密码（明文）
                    client_id: Turtle.isInApp ? plus.push.getClientInfo().clientid : '' //个推用
                });
                mobileRegisterModel.excute(function(json) {
                    this.hideLoading();
                    this.showToast('注册成功');
                    //注册后自动登录
                    getAccessTokenModel.setParam({
                        channel: "user_channel_mobile",
                        username: oAllData.mobileNum,
                        userpwd: this._makeMD5(oAllData.password, this._dateTime)
                    });
                    getAccessTokenModel.excute(function(json) {
                        //登陆就为了获得一个密钥和传入的user手机号
                        var username = oAllData.mobileNum;
                        var secretKey = json.result;
                        headStore.setAttr('username', username);
                        headStore.setAttr('token', secretKey);
                        this.jump(['views/article.html'].join(''));
                    }, function(msg) {
                        this.hideLoading();
                        if (_.isObject(msg) && msg.message) {
                            this.showToast(msg.message, {
                                'hideAction': function() {
                                    this.jump(['views/article.html'].join(''));
                                }
                            });
                            return;
                        }
                        this.showToast('网络错误，请稍候重试。', {
                            'hideAction': function() {
                                this.jump(['views/article.html'].join(''));
                            }
                        });
                    }, false, self);
                }, function(msg) {
                    this.hideLoading();
                    if (_.isObject(msg) && msg.message) {
                        this.showToast(msg.message);
                        return;
                    }
                    this.showToast('网络错误，请稍候重试。');
                }, false, this);
            } else {
                this.showToast(this._errorMsg);
            }
        },
        _isAllValValid: function(oAllData) {
            if (!utilValidate.isMobile(oAllData.mobileNum)) {
                this._errorMsg = '手机号码格式不正确';
                return false;
            }
            if (!utilValidate.isPassword(oAllData.password) ||
                !utilValidate.isPassword(oAllData.passwordRepeat)) {
                this._errorMsg = '密码必须是6-30位字符，可使用字母、数字';
                return false;
            }
            if (oAllData.password !== oAllData.passwordRepeat) {
                this._errorMsg = '两次密码不同哦！';
                return false;
            }
            return true;
        },
        getAllVal: function() {
            var mobileNum = this.$el.find('.js_mobile').val();
            mobileNum = parseInt(mobileNum);
            var securityCode = this.$el.find('.js_code').val();
            securityCode = $.trim(securityCode);
            var password = this.$el.find('.js_pwd').val();
            var passwordRepeat = this.$el.find('.js_pwd_repeat').val();
            return {
                mobileNum: mobileNum || null,
                securityCode: securityCode || null,
                password: password || null,
                passwordRepeat: passwordRepeat || null
            }
        },
        _makeMD5: function(password, sDate) {
            return utilEncryption.md5(utilEncryption.md5(password) + sDate);
        },
        countDown: function(nLeftTime) {
            var self = this;
            var zpItem = self.$el.find('.js_send_code');

            function _countDown() {
                if (nLeftTime >= 0) {
                    minutes = Math.floor(nLeftTime / 60);
                    seconds = Math.floor(nLeftTime % 60);
                    zpItem.text(nLeftTime + '秒后重新发送');
                    --nLeftTime;
                } else {
                    clearInterval(timer);
                    zpItem.removeClass('disable');
                    zpItem.text('重新获取');
                }
            }
            timer = setInterval(function() {
                _countDown();
            }, 1000);
        },
        updateHeader: function() {
            var self = this;
            this.headerView.set({
                view: this,
                leftHandler: [{
                    className: "icon closes",
                    event: function(event) {
                        self.back();
                    }
                }],
                centerHandler: [{
                    title: "手机注册",
                    className: "title"
                }],
                rightHandler: [{

                }]
            });
            this.headerView.show();
        }
    });

    this.viewIntance = view.getIntance();
});