define(['BaseView', 'Model', 'Store', 'text!TemplatesResetPassword', 'utilValidate', 'utilEncryption'], function(BaseView, models, store, tplResetPassword, utilValidate, utilEncryption) {

    var getAccessTokenModel = models.getAccessTokenModel;
    var getConfigModel = models.getConfigModel;
    var sendSmsForgetModel = models.sendSmsForgetModel;
    var forgetPasswordModel = models.forgetPasswordModel;
    var headStore = store.headStore;

    var view = BaseView.extend({
        viewName: 'reset_password',
        _errorMsg: null,
        _dateTime: null, //从 sys_datetime 获取用做加密用
        events: {
            'click .js_login': 'resetPassword',
            'click .js_resetcode': 'sendSecurityCode',
        },
        render: function(html) {
            this.$el.html(html);
        },
        onShow: function() {
            this.render(_.template(tplResetPassword)());
            this.getSiteConfig();
        },
        getSiteConfig: function() {
            this.showLoading();
            getConfigModel.setParam({
                client: 1
            });
            getConfigModel.excute(function(json) {
                this.hideLoading();
                //服务器返回的 "2015-06-24 11:08:53" 就用他们原来的代码改了
                var sysDatetime = json.result.sys_datetime;
                var aDateTime = sysDatetime.replace(" ", ":").replace(/\:/g, "-").split("-");
                this._dateTime = aDateTime[1] + aDateTime[2];
            }, function(msg) {
                this.hideLoading();
                if (_.isObject(msg) && msg.message) {
                    this.showToast(msg.message);
                    return;
                }
                this.showToast('网络错误，请稍候重试。');
            }, false, this);
        },
        resetPassword: function() {
            var self = this;
            var oAllData = this.getAllVal();
            if (this._isAllValValid(oAllData)) {
                this.showLoading();
                forgetPasswordModel.setParam({
                    channel: "user_channel_mobile", // varchar(255) 注册渠道
                    username: oAllData.mobileNum,
                    code: oAllData.securityCode, // char(4)      手机短信验证码
                    userpwd: oAllData.password, // varchar(6-20)    密码（明文）
                    client_id: Turtle.isInApp ? plus.push.getClientInfo().clientid : '' //个推用
                });
                forgetPasswordModel.excute(function(json) {
                    this.hideLoading();
                    //自动登录
                    getAccessTokenModel.setParam({
                        channel: "user_channel_mobile",
                        username: oAllData.mobileNum,
                        userpwd: this._makeMD5(oAllData.password, this._dateTime)
                    });
                    getAccessTokenModel.excute(function(json) {
                        //登陆就为了获得一个密钥和传入的user手机号
                        var username = oAllData.mobileNum;
                        var secretKey = json.result;
                        headStore.setAttr('username', username);
                        headStore.setAttr('token', secretKey);
                        this.showToast('设置成功', {
                            'hideSec': 1500,
                            'hideAction': function() {
                                self.jump(['views/article.html'].join(''));
                            }
                        });
                    }, function(msg) {
                        this.hideLoading();
                        if (_.isObject(msg) && msg.message) {
                            this.showToast(msg.message, {
                                'hideAction': function() {
                                    this.jump(['views/article.html'].join(''));
                                }
                            });
                            return;
                        }
                        this.showToast('网络错误，请稍候重试。', {
                            'hideAction': function() {
                                this.jump(['views/article.html'].join(''));
                            }
                        });
                    }, false, self);
                }, function(msg) {
                    this.hideLoading();
                    if (_.isObject(msg) && msg.message) {
                        // this.showToast(msg.message);
                        $(".login_info").html(msg.message);
                        return;
                    }
                    this.showToast('网络错误，请稍候重试。');
                }, false, this);
            } else {
                $(".login_info").html(this._errorMsg);
                // this.showToast(this._errorMsg);
            }
        },
        _makeMD5: function(password, sDate) {
            return utilEncryption.md5(utilEncryption.md5(password) + sDate);
        },
        _isAllValValid: function(oAllData) {
            var self = this;
            if (!utilValidate.isMobile(oAllData.mobileNum)) {

                var infoItem = self.$el.find('.login_info');
                infoItem.text("手机号码格式不正确");
                this._errorMsg = '手机号码格式不正确';
                return false;
            }
            if (!oAllData.securityCode) {
                $(".login_info").html("请输入验证码");
                $(".js_resetcode").css('background-image',"url("+Turtle.baseUrl+"/../resource/images/button_2.png)");
                this._errorMsg = '请输入验证码';
                return false;
            }


            if (!oAllData.password) {
                $(".login_info").html("请输入密码");
                this._errorMsg = '请输入密码';
                return false;
            }
            if (!utilValidate.isPassword(oAllData.password) ) {
                 $(".login_info").html("密码必须是6-30位字符，可使用字母、数字");
                this._errorMsg = '密码必须是6-30位字符，可使用字母、数字';
                return false;
            }
            return true;
        },
        /**
         * 发送验证码
         */
        sendSecurityCode: function(e) {
            var zpItem = $(e.currentTarget);
            var oAllData = this.getAllVal();
            if (zpItem.hasClass('disable')) {
                return;
            }
            $(".js_resetcode").css('background-image',"url("+Turtle.baseUrl+"/../resource/images/button_1.png)");
            this.showLoading();
            sendSmsForgetModel.setParam({
                username: oAllData.mobileNum
            });
            sendSmsForgetModel.excute(function(json) {
                this.hideLoading();
                // this.showToast('验证码发送成功');
                $(".login_info").html("验证码发送成功");
                zpItem.addClass('disable');
                this.countDown(60);
            }, function(msg) {
                this.hideLoading();
                if (_.isObject(msg) && msg.message) {
                    $(".login_info").html(msg.message);
                    // this.showToast();
                    return;
                }
                this.showToast('网络错误，请稍候重试。');
            }, false, this);

        },
        countDown: function(nLeftTime) {
            var self = this;
            var zpItem = self.$el.find('.js_resetcode');

            function _countDown() {
                if (nLeftTime >= 0) {
                    minutes = Math.floor(nLeftTime / 60);
                    seconds = Math.floor(nLeftTime % 60);
                    zpItem.text(nLeftTime + '秒');
                    --nLeftTime;
                } else {
                    clearInterval(timer);
                    zpItem.removeClass('disable');
                    zpItem.text('重新获取');
                }
            }
            timer = setInterval(function() {
                _countDown();
            }, 1000);
        },
        getAllVal: function() {
            var mobileNum = this.$el.find('.js_mobile').val();
            mobileNum = parseInt(mobileNum);
            var password = this.$el.find('.js_pwd').val();
            var securityCode = this.$el.find('.js_code').val();
            securityCode = $.trim(securityCode);
            return {
                mobileNum: mobileNum || null,
                securityCode: securityCode || null,
                password: password || null
            }
        },
        updateHeader: function() {
            var self = this;
            this.headerView.set({
                view: this,
                leftHandler: [{
                    // className: "icon closes",
                    className: "icon back2",
                    event: function(event) {
                        self.back();
                    }
                }],
                centerHandler: [{
                    title: "忘记密码",
                    className: "title"
                }],
                rightHandler: [{

                }]
            });
            this.headerView.show();
        }
    });

    this.viewIntance = view.getIntance();
});