define(['BaseView', 'Model', 'Store', 'text!TemplatesSafety', 'util', 'utilValidate', 'utilEncryption'], function(BaseView, models, store, tplSafety, util, utilValidate, utilEncryption) {

    var isLogin = util.isLogin();
    var resetPwdModel = models.resetPwdModel;
    var getAccessTokenModel = models.getAccessTokenModel;
    var getConfigModel = models.getConfigModel;
    var headStore = store.headStore;

    if (isLogin) {
        resetPwdModel.url = resetPwdModel.url + util.getToken(); //他们的接口有问题必须这么写 不能用access_token
    }

    var view = BaseView.extend({
        viewName: 'safety',
        _dateTime: null, //从 sys_datetime 获取给加密用
        _errorMsg: null,
        events: {
            'click .js_resetPwd': 'resetPwd'
        },
        render: function(html) {
            this.$el.html(html);
        },
        onShow: function() {
            if (!isLogin) {
                this.jump(['views/login_phone.html', '?from=views/', this.viewName, '.html'].join(''));
                return;
            }
            this.render(_.template(tplSafety)({
                username: util.getUserName()
            }));
            this.getSiteConfig();
        },
        getSiteConfig: function() {
            this.showLoading();
            getConfigModel.setParam({
                client: 1
            });
            getConfigModel.excute(function(json) {
                this.hideLoading();
                //服务器返回的 "2015-06-24 11:08:53" 就用他们原来的代码改了
                var sysDatetime = json.result.sys_datetime;
                var aDateTime = sysDatetime.replace(" ", ":").replace(/\:/g, "-").split("-");
                this._dateTime = aDateTime[1] + aDateTime[2];
            }, function(msg) {
                this.hideLoading();
                if (_.isObject(msg) && msg.message) {
                    this.showToast(msg.message);
                    return;
                }
                this.showToast('网络错误，请稍候重试。');
            }, false, this);
        },
        resetPwd: function(e) {
            var self = this;
            var oAllData = this.getAllVal();
            if (this._isAllValValid(oAllData)) {
                this.showLoading();
                resetPwdModel.setParam({
                    username: oAllData.mobileNum,
                    userpwd: this._makeMD5(oAllData.curPwd, this._dateTime),
                    new_userpwd: oAllData.newPwd
                });
                resetPwdModel.excute(function(json) {
                    this.hideLoading();
                    this.showToast('修改成功', {
                        'hideAction': function() {
                            //修改后自动登录
                            getAccessTokenModel.setParam({
                                channel: "user_channel_mobile",
                                username: oAllData.mobileNum,
                                userpwd: self._makeMD5(oAllData.newPwd, self._dateTime)
                            });
                            getAccessTokenModel.excute(function(json) {
                                //登陆就为了获得一个密钥和传入的user手机号
                                var username = oAllData.mobileNum;
                                var secretKey = json.result;
                                headStore.setAttr('username', username);
                                headStore.setAttr('token', secretKey);
                                this.jump(['views/article.html'].join(''));
                            }, function(msg) {
                                this.hideLoading();
                                if (_.isObject(msg) && msg.message) {
                                    this.showToast(msg.message, {
                                        'hideAction': function() {
                                            this.jump(['views/article.html'].join(''));
                                        }
                                    });
                                    return;
                                }
                                this.showToast('网络错误，请稍候重试。', {
                                    'hideAction': function() {
                                        this.jump(['views/article.html'].join(''));
                                    }
                                });
                            }, false, self);
                        }
                    });
                }, function(msg) {
                    this.hideLoading();
                    if (_.isObject(msg) && msg.message) {
                        this.showToast(msg.message);
                        return;
                    }
                    this.showToast('网络错误，请稍候重试。');
                }, false, this);

            } else {
                this.showToast(this._errorMsg);
            }
        },
        _isAllValValid: function(oAllData) {
            if (!utilValidate.isMobile(oAllData.mobileNum)) {
                this._errorMsg = '手机号码格式不正确';
                return false;
            }
            if (!utilValidate.isPassword(oAllData.newPwd) ||
                !utilValidate.isPassword(oAllData.newPwdRepeat)) {
                this._errorMsg = '新密码必须是6-30位字符，可使用字母、数字';
                return false;
            }
            if (oAllData.newPwd !== oAllData.newPwdRepeat) {
                this._errorMsg = '两次密码不同哦！';
                return false;
            }
            return true;
        },
        getAllVal: function() {
            var mobileNum = this.$el.find('.js_mobile').val();
            mobileNum = parseInt(mobileNum);
            var curPwd = this.$el.find('.js_curPwd').val();
            var newPwd = this.$el.find('.js_newPwd').val();
            var newPwdRepeat = this.$el.find('.js_newPwdRepeat').val();
            return {
                mobileNum: mobileNum || null,
                curPwd: curPwd || null,
                newPwd: newPwd || null,
                newPwdRepeat: newPwdRepeat || null
            }
        },
        _makeMD5: function(password, sDate) {
            return utilEncryption.md5(utilEncryption.md5(password) + sDate);
        },
        updateHeader: function() {
            var self = this;
            this.headerView.set({
                view: this,
                leftHandler: [{
                    className: "icon back",
                    event: function(event) {
                        self.back();
                    }
                }],
                centerHandler: [{
                    title: "设置",
                    className: "title"
                }],
                rightHandler: [{

                }]
            });
            this.headerView.show();
        }
    });

    this.viewIntance = view.getIntance();
});